from .info import __doc__
from . import administration
from . import application
from . import wave_theory
from . import signal_analysis
from . import miscellaneous
from . import software

__version__ = "1.0.0"

